function [invDh] = EigenFeatureReg(D,rt)
% Eigen feature regularization using Jiang et al. 2008, PAMI paper
%
% invDh = EigenFeatureReg(D,rt)
%
% INPUT  : D eigenvalues (from SVD computation);
%          rt rank of total-scatter matrix
% OUTPUT : invDh estimated inverse eigenvalues

rw = rank(D);
dw = diag(D);
% Determining the value of m
lamda = dw.^2;%inv_dw;
lamda_median = median(lamda);
lamda_r = min(lamda);

for j=1:rw
    if lamda(j) < 2*lamda_median-lamda_r
        m=j-1;
        break
    end
end


% Regularized eigensepctrum
lamda_h = zeros(1,rw+1);
alpha = (lamda(1)*lamda(m)*(m-1))/(lamda(1)-lamda(m));
beta  = (m*lamda(m)-lamda(1))/(lamda(1)-lamda(m));
lamda_h(1:m-1) = lamda(1:m-1); % k<m
k=m:rw;
len = length(k);
lamda_h(m:rw) = alpha./(k+beta*ones(1,len)); % m <= k <= r
lamda_h(rw+1:rt) = ones(1,rt-rw)*alpha/(rw+1+beta); % r < k <= n
wk = 1./sqrt(lamda_h);
invDh = diag(wk);%diag(lamda_h);

end

